

local moduleName = "HealthBar"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Health Bar",

	["Agro Color"] = true,
	["Color the healthbar of the person with agro red."] = true,

	["Texture"] = true,
	["The texture used for the health bar."] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local SharedMedia     = AceLibrary("SharedMedia-1.0")
local SharedMediaType = SharedMedia.MediaType and SharedMedia.MediaType.STATUSBAR or "statusbar"

local texture



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.options = {
	name = L[moduleName],
	args = {
		agrocolor = {
			type     = "toggle",
			name     = L["Agro Color"],
			desc     = L["Color the healthbar of the person with agro red."],
			disabled = function() return plugin.agromodule == nil end,
            get      = "GetSetColorAgro",
            set      = "GetSetColorAgro"
		},
		texture = {
			type = 'text',
			name = L["Texture"],
			desc = L["The texture used for the health bar."],
			get  = "GetSetTexture",
			set  = "GetSetTexture",
			validate = SharedMedia:List(SharedMediaType),
		}
	}
}

plugin.defaultDB = {
	position = 2,

	agrocolor = true,
	texture   = "Charcoal"
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	if self.core:HasModule("Agro") then
		self.agromodule = self.core:GetModule("Agro")
	end

	texture = SharedMedia:Fetch(SharedMediaType, self.db.profile.texture)

	--self:ScheduleRepeatingEvent("BunchOfBars_HealthBarUpdate", self.Update, 0.75, self)
	self:RegisterBucketEvent({"UNIT_HEALTH", "UNIT_MAXHEALTH"}, 0.05, "UpdateUnits")
end


function plugin:UpdateBar(bar, unit)
	local m = UnitHealthMax(unit)
	local c = UnitHealth(unit)

	bar:SetMinMaxValues(0, m)
	bar:SetValue(c)

	local agro
	if self.db.profile.agrocolor and self.agromodule then
		agro = self.agromodule.agro[UnitName(unit)]
	end

	if agro then
		if agro < 1.5 then
			bar:SetStatusBarColor(1, 0.5, 0)
		else
			bar:SetStatusBarColor(1, 0, 0)
		end
	else
		local p = c / m
		local r, g

		if p >= 0.5 then
			r, g = (1 - p) * 2, 1
		else
			r, g = 1, p * 2
		end

		bar:SetStatusBarColor(r, g, 0)
	end
end


function plugin:OnCreate(frame)
	local bar = CreateFrame("StatusBar", nil, frame)
	bar:SetStatusBarTexture(texture)
	bar:SetWidth(100)
	bar:SetHeight(18)

	local back = frame:CreateTexture(nil, "OVERLAY")
	back:SetTexture("Interface/Tooltips/UI-Tooltip-Background")
	back:SetVertexColor(0.5, 0.5, 0.5, 1)
	back:ClearAllPoints()
	back:SetAllPoints(bar)
	back:Show()

	return bar
end


function plugin:OnUpdate(frame, bar)
	self:UpdateBar(bar, frame.unit)
end


--[[function plugin:Update()
	for unit,frame in pairs(self.core.frames) do
		self:UpdateBar(frame.parts[self.name], unit)
	end
end]]--


function plugin:UpdateUnits(units)
	for unit in pairs(units) do
        if self.core.frames[unit] then
			self:UpdateBar(self.core.frames[unit].parts[self.name], unit)
		end
    end
end


function plugin:GetSetColorAgro(v)
	if type(v) == "nil" then return self.db.profile.agrocolor end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.agrocolor ~= v then
		self.db.profile.agrocolor = v

		-- TODO: disable the agro module if needed
		
		self:OnEnable()
	end
end


function plugin:GetSetTexture(v)
	if type(v) == "nil" then return self.db.profile.texture end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.texture ~= v then
		self.db.profile.texture = v

		texture = SharedMedia:Fetch(SharedMediaType, self.db.profile.texture)
		
		for _,frame in pairs(self.core.frames) do
			frame.parts[self.name]:SetStatusBarTexture(texture)
		end
	end
end
