

local moduleName = "HealthBarText"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Health Bar Text",

	["Text Format"] = true,

	["Predefined Formats"] = true,
	["Choose from a set of prefedined formats."] = true,

	["Text Justification"] = true,
	["Set the horizontal text justification."] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local DogTag = AceLibrary("DogTag-1.0")

local formats = {
	"[[HasDivineIntervention:[Dead?IsFeignedDeath:HasSoulstone:Dead]]:[IsMouseOver?Text([MissingHP:Negate:Red] | [CurHP:Green])]]",
	"[IsMouseOver?Text([MissingHP:Negate:Red] | [CurHP:Green])]",
	"[HasDivineIntervention:[Dead?IsFeignedDeath:HasSoulstone:Dead]]"
}



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.options = {
	name = L[moduleName],
	args = {
		format = {
			type  = "text",
			name  = L["Text Format"],
			desc  = L["Text Format"],
			usage = "DogTag-1.0 tags. See http://www.wowace.com/wiki/DogTag-1.0",
			get   = "GetSetFormat",
			set   = "GetSetFormat"
		},
		preformats = {
			type     = "text",
			name     = L["Predefined Formats"],
			desc     = L["Choose from a set of prefedined formats."],
			usage    = "",
			validate = formats,
			get      = "GetSetFormat",
			set      = "GetSetFormat"
		},
		justification = {
			type     = "text",
			name     = L["Text Justification"],
			desc     = L["Set the horizontal text justification."],
			usage    = "",
			validate = { "LEFT", "CENTER", "RIGHT" },
			get      = "GetSetTextJustification",
			set      = "GetSetTextJustification"
		},
	}
}

plugin.defaultDB = {
	format        = formats[1],
	justification = "CENTER"
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnEnable()
	self:RegisterEvent("UNIT_MAXHEALTH")
end


function plugin:OnCreate(frame)
	local bar  = frame.parts["HealthBar"]
	local font = bar:CreateFontString(nil, "OVERLAY")
	font:SetFontObject(GameFontHighlightSmall)
	font:SetVertexColor(1, 1, 1, 1)
	font:ClearAllPoints()
	font:SetPoint("BOTTOMLEFT", bar, "BOTTOMLEFT", 2, 0)
	font:SetPoint("TOPRIGHT", bar, "TOPRIGHT", -2, 0)
	font:Show()

	return font
end


function plugin:OnUpdate(frame, font)
	font:SetJustifyH(self.db.profile.justification)

	DogTag:RegisterFontString(font, frame, frame.unit, self.db.profile.format)
end


function plugin:OnInactive(frame, mypart)
	DogTag:UnregisterFontString(mypart)
end


-- stupix fix for removing the "Dead" of alive people and it doesn't even work most of the time
function plugin:UNIT_MAXHEALTH(unit)
	if self.core.frames[unit] then
		DogTag:UpdateFontString(self.core.frames[unit].parts[self.name])
	end
end


function plugin:GetSetFormat(v)
	if type(v) == "nil" then return self.db.profile.format end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.format ~= v then
		self.db.profile.format = v

		self:UpdateAll()
	end
end


function plugin:GetSetTextJustification(v)
	if type(v) == "nil" then return self.db.profile.justification end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.justification ~= v then
		self.db.profile.justification = v

		self:UpdateAll()
	end
end
