

----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBarsLayout")

L:RegisterTranslations("enUS", function() return {
	["Update"] = true,
	["Update everything."] = true,

	["Reset Position"] = true,
	["Move BunchOfBars to the middle of the screen."] = true,

	["Padding"] = true,
	["Padding between module parts."] = true,
	
	["Scale"] = true,
	["The scale of the frames."] = true,

	["Always"] = true,
	["Out Of Combat"] = true,
	["Never"] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local DogTag          = AceLibrary("DogTag-1.0")
local SharedMedia     = AceLibrary("SharedMedia-1.0")
local SharedMediaType = SharedMedia.MediaType and SharedMedia.MediaType.STATUSBAR or "statusbar"


local tooltip = {
	[1] = L["Always"],
	[2] = L["Out Of Combat"],
	[3] = L["Never"]
}

local tooltiprev = { } -- reverse lookup of tooltip
do
	for k,v in ipairs(tooltip) do
		tooltiprev[v] = k
	end
end


local order



----------------------------------
--      Module Declaration      --
----------------------------------

BunchOfBars.options.args.visual.args = {
	update = {
		type = "execute",
		name = L["Update"],
		desc = L["Update everything."],
		func = "UpdateAll"
	},
	reset = {
		type = "execute",
		name = L["Reset Position"],
		desc = L["Move BunchOfBars to the middle of the screen."],
		func = "ResetPosition"
	},
	padding = {
		type = "range",
		name = L["Padding"],
		desc = L["Padding between module parts."],
		min  = 0,
		max  = 15,
		step = 1,
		get  = "GetSetPadding",
		set  = "GetSetPadding",
	},
	scale = {
		type = "range",
		name = L["Scale"],
		desc = L["The scale of the frames."],
		min  = 0.5,
		max  = 2,
		step = 0.1,
		get  = "GetSetScale",
		set  = "GetSetScale",
	},
	tooltip = {
		type     = "text",
		name     = "Show Tooltip",
		desc     = "When to show the unit's tooltip.",
		usage    = "",
		validate = tooltip,
		get      = "GetSetToolTip",
		set      = "GetSetToolTip"
	}
}


BunchOfBars.defaults.visual = {
	scale = 1.0,
	locked = false,
	showtooltip = 2, -- Out Of Combat
	padding = 5,
	x = 400,
	y = 400
}



----------------------------------
--      Module Functions        --
----------------------------------

function BunchOfBars:InitializeLayout()
	ClickCastFrames = ClickCastFrames or {}

	self.frames = {}

	SharedMedia:Register(SharedMediaType, "Charcoal", "Interface\\Addons\\BunchOfBars\\Textures\\Charcoal")
end


function BunchOfBars:OnProfileEnable() -- will be called when our profile is reset
	self:UpdateAll()
end


function BunchOfBars:OnEnable(first)
	--[[
	for i = 1, 4 do
		local frame = _G["PartyMemberFrame"..i]
		frame:UnregisterAllEvents()
		frame:Hide()
		frame.Show = function() end
	end
	--]]

	if not first then return end

	self:RegisterModules()

	self:UpdateLayoutOrder()

	-- The point at which we create this frame seems important, if we do it to early some stuff doesn't work.
	self:ScheduleEvent(self.CreateMaster, 1, self)
end


function BunchOfBars:CreateMaster()
	local frame = CreateFrame("Frame", "BunchOfBarsFrame", UIParent, "SecureGroupHeaderTemplate")
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:SetScale(self.db.profile.visual.scale)

	frame:SetHeight(1)
	frame:SetWidth(1)

	local s = frame:GetEffectiveScale()
	local x, y = BunchOfBars.db.profile.visual.x / s, BunchOfBars.db.profile.visual.y / s

	frame:ClearAllPoints()
	frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)


	--[[
	nameList = [STRING] -- a comma separated list of player names (not used if 'groupFilter' is set)
	groupFilter = [1-8, STRING] -- a comma seperated list of raid group numbers and/or uppercase class names
	strictFiltering = [BOOLEAN] - if true, then characters must match both a group and a class from the groupFilter list
	point = [STRING] -- a valid XML anchoring point (Default: "TOP")
	xOffset = [NUMBER] -- the x-Offset to use when anchoring the unit buttons (Default: 0)
	yOffset = [NUMBER] -- the y-Offset to use when anchoring the unit buttons (Default: 0)
	sortMethod = ["INDEX", "NAME"] -- defines how the group is sorted (Default: "INDEX")
	sortDir = ["ASC", "DESC"] -- defines the sort order (Default: "ASC")
	template = [STRING] -- the XML template to use for the unit buttons
	templateType = [STRING] - specifies the frame type of the managed subframes (Default: "Button")
	columnSpacing
	columnAnchorPoint
	initial-width
	initial-height
	initial-scale
	initial-unitWatch has to do with RegisterUnitWatch
	groupBy = ["GROUP", "CLASS"]
	groupingOrder = [1-8, STRING] -- a comma seperated list of raid group numbers and/or uppercase class names
	maxColumns
	unitsPerColumn
	--]]
	frame:SetAttribute("showRaid", true)
	frame:SetAttribute("showParty", true)
	frame:SetAttribute("showPlayer", true)
	frame:SetAttribute("showSolo", true)

	frame:SetAttribute("yOffset", -1) -- space between frames

	frame:SetAttribute("groupBy", "CLASS")
	frame:SetAttribute("groupingOrder", "WARRIOR,DRUID,PALADIN,PRIEST,SHAMAN,ROGUE,MAGE,WARLOCK,HUNTER")

	frame.initialConfigFunction = self.ConfigureFrame
	frame:SetAttribute("template", "SecureUnitButtonTemplate")
	
	self.master = frame -- self.frame is used by FuBarPlugin
	frame:Show() -- direct after this show it will start creating unit frames
end


local function OnAttributeChanged(frame, name, value)
	if name ~= "unit" then return end -- we are only interrested in the unit attribute

	if frame.unit and not value and frame.created then
		BunchOfBars:ModuleOnInactive(frame)

		BunchOfBars.frames[frame.unit] = nil
		frame.unit = nil
	elseif value then
		if frame.unit then BunchOfBars.frames[frame.unit] = nil end
		frame.unit = value
		BunchOfBars.frames[frame.unit] = frame

		BunchOfBars:ModuleOnUpdate(frame)
		BunchOfBars.master:StopMovingOrSizing() -- seems to fix a bug where you can't stop dragging
	end
end


local function OnDragStart()
	if not BunchOfBars.db.profile.visual.locked and not InCombatLockdown() then
		BunchOfBars.master:StartMoving()
	end
end

local function OnDragStop()
	BunchOfBars.master:StopMovingOrSizing()

	local x,y = BunchOfBars.master:GetLeft(), BunchOfBars.master:GetTop()
	local s   = BunchOfBars.master:GetEffectiveScale()

	BunchOfBars.db.profile.visual.x = x*s
	BunchOfBars.db.profile.visual.y = y*s
end


local function OnEnter()
	if BunchOfBars.db.profile.visual.showtooltip == 1 or -- Always
	   (BunchOfBars.db.profile.visual.showtooltip == 2 and not InCombatLockdown()) then -- Out Of Combat
		UnitFrame_OnEnter()
	end

	DogTag:OnMouseoverUpdate()
end

local function OnLeave()
	UnitFrame_OnLeave()

	DogTag:OnMouseoverUpdate()
end


function BunchOfBars.ConfigureFrame(frame)
	frame:SetAttribute("*type1", "target")

	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:RegisterForDrag("LeftButton")

	frame:SetScript("OnDragStart", OnDragStart)
	frame:SetScript("OnDragStop", OnDragStop)
	frame:SetScript("OnEnter", OnEnter)
	frame:SetScript("OnLeave", OnLeave)
	frame:SetScript("OnAttributeChanged", OnAttributeChanged)

	frame:SetAlpha(1.0)

	BunchOfBars:ModuleOnCreate(frame)
	BunchOfBars:UpdateLayout(frame) -- apperantly we are able to change the size of the frame here, even in combat

	ClickCastFrames[frame] = true
end


function BunchOfBars:ModuleOnCreate(frame)
	frame.parts = {}

	for name,module in self:IterateModules() do
		frame.parts[name] = module:OnCreate(frame)
	end
end

function BunchOfBars:ModuleOnUpdate(frame)
	for name,module in self:IterateModules() do
		module:OnUpdate(frame, frame.parts[name])
	end
end

function BunchOfBars:ModuleOnInactive(frame)
	for name,module in self:IterateModules() do
		module:OnInactive(frame, frame.parts[name])
	end
end


function BunchOfBars:UpdateLayoutOrder()
	order = {}

	for name,module in self:IterateModules() do
		if module.db.profile.position and module.db.profile.position > 0 then
			local done = false
			for k,v in ipairs(order) do
				if module.db.profile.position < self:GetModule(v).db.profile.position then
					table.insert(order, k, name)
					done = true
					break
				end
			end

			if not done then
				table.insert(order, name)
			end
		end
	end
end


function BunchOfBars:UpdateLayout(frame)
	local last = nil
	local width = 0

	-- Hide all parts
	for name,module in self:IterateModules() do
		if module.db.profile.position then
			frame.parts[name]:Hide()
		end
	end

	-- Order and show them
	for _,name in ipairs(order) do
		local p = frame.parts[name]

		width = width + frame.parts[name]:GetWidth() + self.db.profile.visual.padding

		p:ClearAllPoints()
		p:SetPoint("TOP", frame, "TOP", 0, (p:GetHeight() - 20) / 4)

		if not last then
			p:SetPoint("LEFT", frame, "LEFT", self.db.profile.visual.padding, 0)
		else
			p:SetPoint("LEFT", last, "RIGHT", self.db.profile.visual.padding, 0)
		end

		p:Show()

		last = p
	end

	frame:SetWidth(math.ceil(width) + 1)
	frame:SetHeight(20)

	frame:SetScale(BunchOfBars.db.profile.visual.scale)
end

function BunchOfBars:UpdateLayouts()
	for _,frame in pairs(self.frames) do
		self:UpdateLayout(frame)
	end
end


function BunchOfBars:UpdateAll()
	for _,frame in pairs(self.frames) do
		self:ModuleOnUpdate(frame)
		self:UpdateLayout(frame)
	end
end


function BunchOfBars:ResetPosition()
	self.master:ClearAllPoints()
	self.master:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
end




BunchOfBars.modulePrototype.core = BunchOfBars
BunchOfBars.modulePrototype.defaultDB = {}


function BunchOfBars.modulePrototype:OnCreate(frame)
	return nil -- do nothing
end

function BunchOfBars.modulePrototype:OnUpdate(frame, mypart)
	-- do nothing
end

function BunchOfBars.modulePrototype:OnInactive(frame, mypart)
	-- do nothing
end


function BunchOfBars.modulePrototype:UpdateAll()
	for _,frame in pairs(self.core.frames) do
		self:OnUpdate(frame, frame.parts[self.name])
	end
end





function BunchOfBars:GetSetPadding(value)
	if type(value) == "nil" then return self.db.profile.visual.padding end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.visual.padding ~= value then
		self.db.profile.visual.padding = value
		
		self:UpdateLayouts()
	end
end



function BunchOfBars:GetSetScale(value)
	if type(value) == "nil" then return self.db.profile.visual.scale end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.visual.scale ~= value then
		self.db.profile.visual.scale = value
		
		self:UpdateLayouts()
	end
end


function BunchOfBars:GetSetToolTip(value)
	if type(value) == "nil" then return tooltip[self.db.profile.visual.showtooltip] end

	self.db.profile.visual.showtooltip = tooltiprev[value]
end
