

local moduleName = "RangeCheck"



----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("BunchOfBars"..moduleName)

L:RegisterTranslations("enUS", function() return {
	[moduleName] = "Range Check",

	["Check frequency"] = true,
	["Frequency in seconds."] = true,

	["Range Check"] = true,
	["Fade out the frames of unit's who aren't near."] = true
} end)



----------------------------------
--      Local Declaration      --
----------------------------------

local BS = AceLibrary("Babble-Spell-2.2")

local InRange



----------------------------------
--      Module Declaration      --
----------------------------------

local plugin = BunchOfBars:NewModule(moduleName, "AceEvent-2.0")

plugin.options = {
	name = L[moduleName],
	args = {
		frequency = {
			type = "range",
			name = L["Check frequency"],
			desc = L["Frequency in seconds."],
			min  = 0.1,
            max  = 5.0,
            step = 0.1,
            get  = "GetSetFrequency",
            set  = "GetSetFrequency",
		},
		check = {
			type = "toggle",
			name = L["Range Check"],
			desc = L["Fade out the frames of unit's who aren't near."],
            get  = "GetSetCheck",
            set  = "GetSetCheck"
		}
	}
}

plugin.defaultDB = {
	frequency = 1.0,
	check     = true
}



----------------------------------
--      Module Functions        --
----------------------------------

function plugin:OnInitialize()
	local class = select(2, UnitClass("player"))

	-- 40 yard range check
	if class == "PRIEST" then
		InRange = function(unit) return IsSpellInRange(BS["Flash Heal"], unit) == 1 end
	elseif class == "DRUID" then
		InRange = function(unit) return IsSpellInRange(BS["Healing Touch"], unit) == 1 end
	elseif class == "PALADIN" then
		InRange = function(unit) return IsSpellInRange(BS["Holy Light"], unit) == 1 end
	elseif class == "SHAMAN" then
		InRange = function(unit) return IsSpellInRange(BS["Healing Wave"], unit) == 1 end
	elseif class == "MAGE" then
		InRange = function(unit) return IsSpellInRange(BS["Remove Lesser Curse"], unit) == 1 end
	else
		InRange = function(unit) return CheckInteractDistance(unit, 4) end
	end
end


function plugin:OnEnable()
	if self.db.profile.check then
		self:ScheduleRepeatingEvent("BunchOfBars_RangeUpdate", self.UpdateAll, self.db.profile.frequency, self)
	end
end


function plugin:OnUpdate(frame)
	if InRange(frame.unit) then -- this will also hide dead and offline players
		frame:SetAlpha(1.0)
	else
		frame:SetAlpha(0.25)
	end
end


function plugin:GetSetFrequency(value)
	if type(value) == "nil" then return self.db.profile.frequency end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.frequency ~= value then
		self.db.profile.frequency = value
		
		if self.db.profile.check then
			self:OnEnable()
		end
	end
end


function plugin:GetSetCheck(v)
	if type(v) == "nil" then return self.db.profile.check end

	-- this check is to assure we don't reschedule the event to often
	if self.db.profile.check ~= v then
		self.db.profile.check = v
		
		if self.db.profile.check then
			self:OnEnable()
		else
			self:CancelScheduledEvent("BunchOfBars_RangeUpdate")
			
			for _,frame in pairs(self.core.frames) do
				frame:SetAlpha(1.0)
			end
		end
	end
end
